function [heartRate ,HR0, sigBuf, flag1, freqRes] = detectHR_1(sumr, sumg, sumb, vidFPS, params)
%                
% INFO:  Heart rate detection (FFT)
% INPUTS:
% > sumr,sumg,sumb   - sum of pixel values inside ROI for RGB components (normalized) [SINGLE]
% > vidFPS           - current video FPS, SINGLE
% > params           - algorithm external parameters:
%   .componentNr     - image component used in analysis, UINT32
%   .minmaxHR        - [min,max] heart rate , UINT32
%   .HRTABLENGTH     - length of heart rate table (for computing mean or median value), UINT32
%   .p_MinPeakProminence - "findpeaks" parameter, SINGLE
%   .type1           - how to analyse hert rate table, 0=median, 1=mean, UINT32
% OUTPUTS:
% > heartRate        - heart rate value (after HR table analysis), UINT32
% > HR0              - current HR value, UINT32
% > sigBuf           - additional output - signal buffer (normalized selected image component), SINGLE, [BUFL x 1]
% > flag1            - flag of buffer overflow - if 1 then buffer is full and HR results are valid, UINT32
% > freqRes          - resolution of HR detection [bpm], SINGLE
% DEPENDICIES:
% COMMENTS:
% > size of the signal buffer is computed as: BUFL = BUFFERLENGTH*targetFPS
% CREATED: 23.03.2018, Jaromir Przybylo (przybylo@agh.edu.pl), R2017b
% LAST MODIFIED: 
%  
%#codegen

% INTERNAL PARAMETERS
% - Cannot be passed as arguments due to how MATLAB Coder works
NFFT = 512;            % length of FFT
BUFFERLENGTH =30;      % length of signal buffer in [s]
targetFPS = 20;        % assumed target FPS
assert((BUFFERLENGTH*targetFPS)>=NFFT)

% Initialization of peristent variables
persistent sigBuffer kiter bufferFullFlag heartRateTab hri HDSP1

if isempty(sigBuffer)
    sigBuffer = zeros(round(BUFFERLENGTH*targetFPS),3);
end
if isempty(kiter)
    kiter = 1;
end
if isempty(bufferFullFlag)
    bufferFullFlag = 0;
end
if isempty(heartRateTab)
    heartRateTab = zeros(round(params.HRTABLENGTH),1);
end
if isempty(hri)
    hri = 1;
end
if isempty(HDSP1)
    HDSP1 = dsp.SpectrumEstimator('FFTLengthSource','property','FFTLength', double(NFFT),'SpectrumType','power','Window','rectangular','FrequencyRange','onesided');
end

% Load filter cooeficients
filtername = 'filterBank_v1.mat';
if coder.target('MATLAB')
    load(filtername); % for code generation use: coder.load
else
    tmp0 = coder.load(filtername);
    Num1 = tmp0.Num1;
end
assert((BUFFERLENGTH*targetFPS)>=length(Num1));

% Add a new sample to the signal buffer           
sigBuffer(kiter,:) = [sumr sumg sumb];
kiter = kiter + 1;
if kiter>size(sigBuffer,1)
    kiter = 1;
    bufferFullFlag = 1;
end

% HR detection
HR0 = NaN;
HR_pxx_val = NaN;        
ydata1 = sigBuffer(:,params.componentNr)';
ydata2=filter(Num1,1,ydata1 - mean(ydata1)); 
%ydata2 = ydata1 - mean(ydata1);
if bufferFullFlag==1    
    pxx = step(HDSP1,ydata2(:));
    f = HDSP1.getFrequencyVector * double(vidFPS);    
    f1=60*f;
    wybor1 = f1>=params.minmaxHR(1)&f1<=params.minmaxHR(2);
    f2=f1(wybor1);
    pxx2=pxx(wybor1);
    if ~isempty(f2)
        [pks1,locs1] = findpeaks(10*log10(pxx2),'MinPeakProminence',params.p_MinPeakProminence);           
        if length(pks1)>1
            [pks1sorted,idsorted] = sort(pks1,'descend');
            idhr = idsorted(1);
            HR0 = f2(locs1(idhr));
            HR_pxx_val = 10*log10(pxx2(locs1(idhr)));
        end
    end
end      

% HR analysis (HR table)
heartRateTab(hri) = HR0;
hri=hri+1;
if hri>length(heartRateTab)
    hri=1;
end
if params.type1==0
    heartRate = uint32(median(heartRateTab));        
else
    heartRate = uint32(mean(heartRateTab));        
end

% Output arguments
sigBuf = single(ydata1(:));
flag1 = uint32(bufferFullFlag);
freqRes = single(60*(single(vidFPS)/NFFT));
