function [bboxPolygon, visiblePoints, faceROI] = faceDetAndTRack_1(RGB)
%                
% INFO:  Face detection and tracking
% INPUTS:
% > RGB             - input RGB image [M*N*3], UINT8
% OUTPUTS:
% > bboxPolygon     - [x1 y1 x2 y2 x3 y3 x4 y4] vector of face polygon, UINT32
% > facePoints      - [K*2] matrix of points coordinates (x,y), SINGLE
% > faceROI         - [row col width height] of detected face, or [0 0 0 0]
%                     if no face is detected, UINT32
% COMMENTS:
% > based on MATLAB documentation example 
%   "Face Detection and Tracking Using Live Video Acquisition"
% CREATED: 23.03.2018, Jaromir Przybylo (przybylo@agh.edu.pl), R2017b
% LAST MODIFIED: 
%  
%
%#codegen

persistent faceDetector pointTracker numPts bboxPoints oldPoints 

if isempty(faceDetector)
    % Create the face detector object.
    faceDetector = vision.CascadeObjectDetector();
end
if isempty(pointTracker)
    % Create the point tracker object.
    pointTracker = vision.PointTracker('MaxBidirectionalError', 2, 'BlockSize', uint32([31 31]));
    %release(pointTracker);
    initialize(pointTracker, single([1 1]), rgb2gray(RGB));
end
if isempty(numPts)
    % number of tracked points
    numPts = 0;
end
if isempty(bboxPoints)
    bboxPoints = double(zeros(4,2));
end
if isempty(oldPoints)
    oldPoints = single([]);
end


% Face detection and tracking
videoFrameGray = rgb2gray(RGB);
visiblePoints = single([]);
if numPts < 10
    % Detection mode.
    bbox = faceDetector.step(videoFrameGray);
    
    if ~isempty(bbox)
        % Find corner points inside the detected region.
        points = detectMinEigenFeatures(videoFrameGray, 'ROI', bbox(1, :));

        % Re-initialize the point tracker.
        xyPoints = points.Location;
        numPts = size(xyPoints,1);
        %release(pointTracker);
        %initialize(pointTracker, xyPoints, videoFrameGray);
        setPoints(pointTracker,single(xyPoints));
        
        % Save a copy of the points.
        oldPoints = xyPoints;
        
        % Convert the rectangle represented as [x, y, w, h] into an
        % M-by-2 matrix of [x,y] coordinates of the four corners. This
        % is needed to be able to transform the bounding box to display
        % the orientation of the face.
        bboxPoints = bbox2points(bbox(1, :));        
    end
    
else
    % Tracking mode.    
    [xyPoints, isFound] = step(pointTracker, videoFrameGray);
    visiblePoints = single(xyPoints(isFound, :));
    oldInliers = oldPoints(isFound, :);
    
    numPts = size(visiblePoints, 1);
    
    if numPts >= 10
        % Estimate the geometric transformation between the old points
        % and the new points.
        [xform, oldInliers, visiblePoints] = estimateGeometricTransform(...
            oldInliers, visiblePoints, 'similarity', 'MaxDistance', 4);
        % Apply the transformation to the bounding box.
        
        bboxPoints = double(transformPointsForward(xform, bboxPoints));

        % Reset the points.
        oldPoints = visiblePoints;
        setPoints(pointTracker, single(oldPoints));
    end    
end

% Convert the box corners into the [x1 y1 x2 y2 x3 y3 x4 y4]
% format required by insertShape.
bboxPolygon = uint32(reshape(bboxPoints', 1, []));
facePoints = single(visiblePoints);


minx = min(single(bboxPolygon(1:2:end)));
miny = min(single(bboxPolygon(2:2:end)));
maxx = max(single(bboxPolygon(1:2:end)));
maxy = max(single(bboxPolygon(2:2:end)));  
faceROI = uint32([minx miny maxx-minx-1 maxy-miny-1]);

            