%% Test harness - face detection and heart rate estimation
%
% CREATED: 23.03.2018, Jaromir Przybylo (przybylo@agh.edu.pl), R2017b
%
% -------------------------------------------------------------------------
% ACADEMIC OR NON-PROFIT ORGANIZATION NONCOMMERCIAL RESEARCH USE ONLY:
%
% Any publications arising from the use of this software, including but
% not limited to academic journal and conference publications, technical
% reports and manuals, must cite at least one of the following works:
% 1) J. Przybyo, E. Katoch, M. Jaboski, P. Augustyniak, 
%    "Distant measurement of plethysmographic signal in various lighting 
%     conditions using configurable frame-rate camera,", Metrol. Meas. Syst.,
%     vol. 23, no. 4, pp. 579592, 2016.
%
% For inquiries about the commercial use please contact : przybylo@agh.edu.pl
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
% INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
% A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
% COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
% IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION 
% WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
% -------------------------------------------------------------------------
%
clear all;close all;clc

%% Parameters
invidfile = '<INPUT VIDEO FILE>';            % input video file
%-Heart rate detection parameters (see description in "detectHR_1.m"
% detectHR_1.m - buffer lengths and targetFPS are set inside m-file,
%                to simplify C/C++ code generation. If vidFPS~=targetFPS
%                the hr estimates will not be correct
filterOrder1         = 64;%128;              % bandpass filter order
params1 = [];
params1.componentNr = uint32(2);
params1.minmaxHR = uint32([50 180]);
params1.HRTABLENGTH = uint32(50);            
params1.p_MinPeakProminence = single(0.3);
params1.type1 = uint32(1);

%% Open video file
vidObj = VideoReader(invidfile);
vidFPS = vidObj.FrameRate;
nFrames = vidObj.Duration*vidObj.FrameRate;

%% Prepare and save filter coefficients
filtername = 'filterBank_v1.mat';
if ~exist(filtername,'file')
    Hd = bandpass_filter_2(double(vidFPS), double(params1.minmaxHR(1)), double(params1.minmaxHR(2)), filterOrder1);
    Num1 = Hd.Numerator;
    save(filtername,'Num1');
end

%% Main run loop
%- simple GUI
videoPlayer  = figure;guih=[];
guih(1)=uicontrol(videoPlayer,'units','normalized','position',[0 0 0.2 0.05],...
 'string','stop','callback','stopCond=0;');  
guih(1)=uicontrol(videoPlayer,'units','normalized','position',[0.3 0 0.2 0.05],...
 'string','pause','callback','if pauseCond==0,pauseCond=1;set(gco,''String'',''continue'');else pauseCond=0;set(gco,''String'',''pause'');end');  
stopCond=1;pauseCond=0;

%- run loop
iter=1;                         % iteration number
while (iter<nFrames & stopCond)    

    if pauseCond==0
        % Read video frame
        vidFrame = readFrame(vidObj);

        % ==============================================================
        % RUN ALGORITHM
        % - face detection and tracking
        [face_bboxPolygon, face_visiblePoints, faceROI] = faceDetAndTRack_1(vidFrame);
        
        % - Compute sum of pixel values inside ROI 
        [sumr, sumb, sumg, roi1] = roiSelection_1(vidFrame, face_bboxPolygon);        
        
        % - opt. face verification (recognition)
        
        % - heart rate detection
        [heartRate, HR0, sigBuf, flag1, freqRes] = detectHR_1(sumr, sumg, sumb, single(vidFPS), params1);
        
        % ==============================================================        
        
        % Visualization       
        subplot(2,2,1);     % face with ROIs and tracked points
        if ~all(faceROI==0)
            %- face ROI
            vidFrame1 = insertShape(vidFrame, 'Polygon', face_bboxPolygon, 'LineWidth', 2);           
            %- heart rate ROI
            vidFrame1 = insertShape(vidFrame1, 'Rectangle', roi1, 'LineWidth', 1, 'Color' , 'red', 'LineWidth', 2);                   
        end
        if ~isempty(face_visiblePoints)
            %- tracked points
            vidFrame1 = insertMarker(vidFrame1, face_visiblePoints, '+', 'Color', 'white');            
        end
        
        imshow(vidFrame1, 'Border', 'tight');
        axis equal;
        title(['Frame ' num2str(iter) ' of ' num2str(nFrames)])                
        
        subplot(2,2,2);     % cropped face ROI        
        if ~all(faceROI==0)         
            faceROIim = imcrop(vidFrame,faceROI);
            imshow(faceROIim)
            title('Face')
        end
        
        subplot(2,2,3:4);   % signal buffer and heart rate  
        plot(sigBuf)
        grid on
        title(['FPS = ' num2str(vidFPS) ', freq. res. = ' num2str(freqRes) ' [bpm]'])
        tmp1 = ylim;
        if flag1
            % when buffer is full and HR estimate is valid - display it
            text(10, tmp1(2)-(tmp1(2)-tmp1(1))*0.1, ['HR = ' num2str(heartRate)], 'Color', 'g', 'FontWeight', 'bold', 'FontSize', 18);
        else
            text(10, tmp1(2)-(tmp1(2)-tmp1(1))*0.1, ['HR = <wait>'], 'Color', 'r', 'FontWeight', 'bold', 'FontSize', 18);
        end
        
        % --
        iter = iter + 1;    
    end
    pause(0.01)
    drawnow
end
close(videoPlayer);

